import java.applet.Applet;
import java.awt.event.*;
import java.awt.*;


public class EnergyFloat {

    /**Here is the main routine.  We want this to be positive in order
       to pass the block.*/

    public static double theta(long S,long X,long Y) {
	double d0=energy(S,X,Y);
	double d1=energyTBP(S);
	return d0-d1;
    }

    /**This is the floating point version of the Energy class.
       The routines listed in the energy class are long and
       complicated and hard to read.  Here we have much simpler
       floating point routines. We can use the debugger to 
       check that the two things agree to high precision*/

    public static double energy(long S,long X,long Y) {
	double[] d=energyTerms(S,X,Y);
	double sum=0;
	for(int i=0;i<10;++i) sum=sum+d[i];
	return sum;
    }

    /**Here are the 10 terms in the energy*/
    public static double[] energyTerms(long S,long X,long Y) {
	double big=65536.0;
	double e=S/big;
	double x=X/big;
	double y=Y/big;
	double[] d={A(e,x),A(e,y),B(e,x),B(e,x),B(e,y),B(e,y),C(e,x,y),C(e,x,y),C(e,x,y),C(e,x,y)};
	return d;
    }

    public static double A(double e,double x) {
	double t=a(x);
	double t1=Math.pow(t,e/2);
	return t1;
    }

    public static double B(double e,double x) {
	double t=b(x);
	double t1=Math.pow(t,e/2);
	return t1;
    }

    public static double C(double e,double x,double y) {
	double t=c(x,y);
	double t1=Math.pow(t,e/2);
	return t1;
    }

    public static double a(double x) {
	double t=(1+x*x)*(1+x*x)/(16*x*x);
	return t;
    }

    public static double b(double x) {
	double t=(1+x*x)/4;
	return t;
    }

    public static double c(double x,double y) {
	double t=(1+x*x)*(1+y*y)/(4*(x*x+y*y));
	return t;
    }


    /**Here are the analogous routines for the TBP*/

    public static double energyTBP(long S) {
	double[] d=energyTermsTBP(S);
	double sum=0;
	for(int i=0;i<10;++i) sum=sum+d[i];
	return sum;
    }
    
    /**Here are the 10 terms in the energy*/
    public static double[] energyTermsTBP(long S) {
	double big=65536.0;
	double e=S/big;
	double x=1.0;
	double y=Math.sqrt(3)/3;
	double[] d={A(e,x),A(e,y),B(e,x),B(e,x),B(e,y),B(e,y),C(e,x,y),C(e,x,y),C(e,x,y),C(e,x,y)};
	return d;
    }

}


    
